function [bestSize, minErr] = optimizeNeighborhoodNN( ...
  neighborhoodSizeVec, simMat, classLabels)
%optimizeNeighborhoodNN -  
% Optimize the neighborhood size for the nearest neighbors classifier,
% using the cross-validation strategy decribed by Hastie, Tibshirani and
% Friedman (HTF) in "The Elements of Statistical Learning", pages 214-216.
% Assumes similarity data are used (not dissimilarity).
%
% USAGE
% [bestSize, minErr] = optimizeNeighborhoodNN(neighborhoodSizeVec, simMat, ...
%      classLabels);
% neighbrohoodSizeVec: vector of neighborhood size to cross-validate.
% simMat: N-by-N matrix of pairwise similarities.
% classLabels: N-by-1 vector of class albesl for each row of simMat.
% bestSize: optimized neighborhood size.
% minErr: cross-validation error corresponding to best neighborhood size.
%
% See also optimizeNeighborhoodLocalSda, optimiseNeighborhoodLocalNC.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, Feb 2007.
%$Id:$
%$DateTime:$
%$Author:$

neighborhoodSizeVec = sort(neighborhoodSizeVec);
nNeighborhoodSizes = length(neighborhoodSizeVec);
[nData, notUsed] = size(simMat); %#ok
maxNeighbors = max( [max(neighborhoodSizeVec) nData]);

neighborIdx = findNearNeighbors(simMat, maxNeighbors, 'similarity');
neighborIdx = adjustSelfSimilarityRank(neighborIdx);
neighborIdx(:,1) = [];

classEst = 10e10 * ones(nData,nNeighborhoodSizes);
neighborClassLabels = classLabels(neighborIdx);
for iSize = 1:nNeighborhoodSizes
  thisSize = neighborhoodSizeVec(iSize);
  classEst(:,iSize) = mode(neighborClassLabels(:,1:thisSize), 2);
end %for iSize
classMat = repmat( classLabels(:), 1, nNeighborhoodSizes );
errVec = sum( classEst ~= classMat );
[minErr, minErrIdx] = min(errVec);
bestSize = neighborhoodSizeVec(minErrIdx);

return