function [bestSize, minErr] = ...
  crossValidateNeighborhoodLocalBda( neighborhoodSizeVec, neighborType, ...
  simMat, featureMat, crossValIdxMat, classLabels, testIdx)
%crossValidateNeighborhoodLocalBda - cross validate local BDA neighborhood.
% Cross-validate the neighborhood size for the local BDA classifier.
% Uses the N-fold cross validation strategy. This function is designed to be
% used in conjunction with readPartitionFile() and with the partition files
% from the Similarity Learning Repository.
%
% Assumes dissimilarity data are used (not similarity).
%
% USAGE
% [bestSize, minErr] = crossValidateNeighborhoodLocalBda( ...
%     neighborhoodSizeVec, neighborType, simMat, featureMat, ...
%     crossValIdxMat, classLabels, testIdx)
% neighbrohoodSizeVec: vector of neighborhood sizes to cross-validate.
% neighborType: string, can be either 'kPerClass' or 'kTotal', but only
% 'kPerClass' is supported for now.
% simMat: N-by-N matrix of pairwise dissimilarities (Euclidean distances
%     between feature vectors).
% featureMat: N-by-F matrix of feature vectors. The i-th row represents the
%     i-th sample described by F features and corresponds to the i-th
%     sample in simMat. 
% crossValIdxMat: N-by-K matrix of indices into the rows of simMat and featureMat,
%    denoting the samples to be used for cross-validation. It is a cell
%    from crossValIdxCell as returned by readPartitionFile().
% classLabels: N-by-1 vector of class labels for each row of featureMat.
% simDomain: vector of possible similarity values.
% testIdx: a vector of indices into the columns of simMat denoting which
%     columns should be reserved for the testing data set. It is a row from
%     testIdxMat as returned by readPartitionFile(), corresponding to
%     the matrix crossValIdxMat.
% bestSize: optimized neighborhood size.
% minErr: cross-validation error rate corresponding to the optimized
%     parameters. 

% Written by Luca Cazzanti, enginner.
% Applied Physics Lab - University of Washington, Feb 2009.
%$Id:$
%$DateTime:$
%$Author:$

%% Preliminaries

% Vector of neighborhood sizes to cross-validate 
actualNeighborhoodSizeVec = unique(neighborhoodSizeVec);
nSizes = length(actualNeighborhoodSizeVec);

% Number of cross-validation folds
nFolds = size(crossValIdxMat,1);

% END of preliminary stuff


%% Cross-validation loops
    
iterCount = 0;
totalErrMat = 0;
for iFold = 1:nFolds %%DEBUG
  
  disp(['Cross-validation fold: ' num2str(iFold)]);
 
  % Get the cross-validation - that is, the "test" sample for this fold
  crossValIdx = crossValIdxMat(iFold,:);
  crossValIdx(crossValIdx == 0) = []; % take out any remaining filler values

  % Iterate through all the "test" samples in this cross-validation fold
  nCrossVals = length(crossValIdx);
  for iCrossVal = 1:nCrossVals

    % get the ith "test" sample's index
    thisCrossValSampleIdx = crossValIdx(iCrossVal);
    thisCrossValClass = classLabels(thisCrossValSampleIdx);
    thisCrossValFeatureVec = featureMat(thisCrossValSampleIdx,:);
    
    % Remove the sample itself from candidate neighbors indeces.
    candidateNeighborIdx = 1:size(simMat,2);
    candidateNeighborIdx(thisCrossValSampleIdx) = [];
    
    % Remove other cross validation "test" points as candidate neighbors
    % by looking through each of them and removing them
    for iCount = 1:nCrossVals
      idx = find( candidateNeighborIdx == crossValIdx(iCount) );
      candidateNeighborIdx(idx) = [];
    end % for iCount

    % Remove the true test samples as candidate neighbors
    % by looking through each of them and removing them
    for iTest = 1:length(testIdx)
      idx = find( candidateNeighborIdx == testIdx(iTest) );
      candidateNeighborIdx(idx) = [];
    end % for iTest

    % Restrict final choice of neighbors within set of candidate neighbors.
    candidateNeighborClassLabels = classLabels(candidateNeighborIdx);
    testPointSimToCandidates = simMat( thisCrossValSampleIdx, candidateNeighborIdx );
    candidateNeighborFeatureMat = featureMat(candidateNeighborIdx,:);
    
    % Get the actual neighbors
    if strcmpi(neighborType, 'kPerClass')
      neighborIdxByClass = findNearNeighborsByClass(testPointSimToCandidates, ...
        length(candidateNeighborClassLabels), candidateNeighborClassLabels, ...
        'dissimilarity');
      % There should be only one element in the cell array, a matrix.
      neighborIdxMat = neighborIdxByClass{1};
    elseif strcmpi(neighborType, 'kTotal')
      neighborIdxMat = findNearNeighbors(testPointSimToCandidates, ...
        length(candidateNeighborClassLabels), 'dissimilarity');
      %error('kTotal neighborhood would be nice, but untested for local BDA');
    else
      error(['Unsupported neighborhood type: ' neighborType]);
    end
      
    % Iterate over the neighborhood sizes
    classEst = -1e6*ones(1,nSizes);
    
    for iSize = 1:nSizes
      
      nNeighbors = actualNeighborhoodSizeVec(iSize);
      actualNeighborIdx = neighborIdxMat(:, 1:nNeighbors);
      actualNeighborIdx = actualNeighborIdx(:).';
      actualNeighborClassLabels = candidateNeighborClassLabels(actualNeighborIdx);
      actualNeighborFeatureMat = candidateNeighborFeatureMat(actualNeighborIdx,:);
      
      classEst(iSize) = classifyLocalBda( thisCrossValFeatureVec, ...
        actualNeighborFeatureMat, actualNeighborClassLabels);
        
    end % for iSize
       
    % Accumulate the count of incorrect classifications for all samples in
    % all cross validation folds
    thisCrossValErrMat = double(classEst ~= thisCrossValClass);
    totalErrMat = totalErrMat + thisCrossValErrMat;
  
    iterCount = iterCount + 1;
    
  end % for iCrossVal

end % for iFold

avgErrMat = totalErrMat / iterCount;

% Get the parameter values corresponding to the minimum error
[minErr, bestSizeIdx] = min(avgErrMat);
bestSize = actualNeighborhoodSizeVec(bestSizeIdx);
