function [errorCount, classEst, posteriors, modelParams, ...
  sameClassErrorCount, sameClassEst, logMarginals] = evaluateTestsLocalSda( ...
  neighborhoodSize, simMat, testIdx, classLabels, simDomain)
%evaluateTestsLocalSda
% Compute the local SDA classification results for test splits. 
% This functions is designed to be used in conjunction with
% readPartitionFile() and with the partition files from the Similarity
% Learning Repository.
%
% Assumes similarity data are used (not dissimilarity) and the centroid
% type is 'maxSumSim'.
%
% USAGE
% [errorCount, classEst, posteriors, modelParams, ...
%     sameClassErrCount, sameClassEst, logMarginals] = evaluateTestsLocalSda( ...
%     neighborhoodSize, simMat, testIdx, classLabels, simDomain)
% neighbrohoodSize: neighborhood size to use.
% simMat: N-by-N matrix of pairwise similarities.
% testIdx: vector of indices into simMat denoting the samples to be tested.
%    It is a row from testIdxMat as returned by redPartitionFile().
% classLabels: N-by-1 vector of class labels for each column of simMat.
% simDomain: vector of possible similarity values.
% errorCount: number of misclassified test samples.
% classEst: M-by-1 vector of class estimates returned by the local SDA
%     classifier, where M is the number of test samples (the length of
%     testIdx).
% posteriors: M-by-1 cell array, each element containing the class
%     posterior proabilities computed by the local SDA classifier.
% modelParams: M-by-1 cell array, each element containing the local SDA
%     model parameters for a test sample. 
% sameClassErrCount: number of misclassified test samples for same-class
%     local SDA (experimental).
% sameClassEst: M-by-1 vector of class estimates returned by the same-class
%     local SDA classifier, where M is the number of test samples
%     (experimental). 
% logMarginals: M-by-1 cell array. Each element is a G-by-G matrix of 
%     log-marginal probabilities as returned by classifyLocalSda()
%
% See also crossValidateNeighborhoodLocalSda, readPartitionFile,
% classifyLocalSda

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, Feb 2007.
%$Id:$
%$DateTime:$
%$Author:$

if isempty(simDomain)
  simDomain = unique(simMat(:));
else
  simDomain = unique(simDomain);
end

neighborIdx = findNearNeighbors(simMat, size(simMat,2), 'similarity');
neighborIdx = adjustSelfSimilarityRank(neighborIdx);
neighborIdx(:,1) = [];

nTests = length(testIdx);    
classEst = -1e6 * ones(nTests,1);
sameClassEst = -1e6 * ones(nTests,1);
trueClass = classLabels(testIdx);
posteriors = cell(nTests, 1);
modelParams = cell(nTests,1);
logMarginals = cell(nTests,1);

% Iterate through all the "test" samples in this cross-validation fold
for iTest = 1:nTests

  thisTestSampleIdx = testIdx(iTest);

  % Pull out the test sample's neighbors
  neighborsToThisTestSample = neighborIdx(thisTestSampleIdx,:);

  % Remove the other test samples from this sample's possible
  % neighbors: this keeps the tests completely disjoint
  for jTest = 1:nTests
    %idx = find( neighborsToThisTestSample == testIdx(jTest) );
    idx = neighborsToThisTestSample == testIdx(jTest);
    neighborsToThisTestSample(idx) = [];
  end % for jTest

  %length(neighborsToThisTestSample)
  
  neighborsToPick = neighborsToThisTestSample(1:neighborhoodSize);
  neighborClassLabels = classLabels(neighborsToPick);
  testPointSim = simMat( thisTestSampleIdx, neighborsToPick );
  neighborSimMat = simMat( :, neighborsToPick );
  neighborSimMat = neighborSimMat( neighborsToPick, : );

  [classEst(iTest) logPosteriors, modelParams{iTest}, ...
    sameClassLogPosteriors, sameClassEst(iTest), logMarginals{iTest}] = ...
    classifyLocalSda(testPointSim, neighborSimMat, ...
    neighborClassLabels, 'maxSumSim', simDomain);
  posteriors{iTest} = exp(logPosteriors);
  %sameClassPosteriors{iTest} = exp(sameClassLogPosteriors);
end %for iTest

errorCount = sum( classEst(:) ~= trueClass(:) );
sameClassErrorCount = sum( sameClassEst(:) ~= trueClass(:) );

return