% run_nfold_crossval
% Test classification algorithms on the data from the Similarity Learning
% Repository. This script automates testing of several SDA-type algorithms
% on the repository data.

%% Prelims

% List of data sets to run: the data for each set are assumed in the
% directory indicated below, in the similarity learning repository format
% See the Similarty Learning Repository website for information on this
% format.

%dataDirList = {'Patrol', 'Protein', 'Voting', 'InternetAds-linear'};
%dataDirList = {'AuralSonar', 'Patrol', 'Protein', 'Voting'};
%dataDirList = {'InternetAds-linear'};
%dataDirList = {'AuralSonar'};
dataDirList = {'Amazon96'};

% Append a string to the result file name to avoid overwriting results
timeStamp = datestr(now, 30); % for uniquely identifying runs

%% Cycle through the directories and run algorithms on the data
for iDir = 1:length(dataDirList);
  
  dataDir = dataDirList{iDir};
  disp(dataDir)

  % Read in similarity data and labels
  filePrefix = fullfile(dataDir,dataDir);
  data = readSimData(filePrefix);

  % The sda-type algorithms have been tested with symmetric symilarities:
  % symmetrize the sim matrix if it is not natively symmetric.
  tmp = data.S.';
  if any(data.S(:) - tmp(:))
    data.S = ( data.S + tmp) / 2; % Force symmetric similarity
  end

  simMat = full(data.S);
  classLabels = data.lbl;
  clear data

  % Correction to the patrol data: make self-similarities be maximum 
  if strcmpi(dataDir, 'Patrol')
    disp('Applying correction to Patrol data self-similarity');
    for iData = 1:size(simMat, 1)
      simMat(iData, iData) = max(simMat(:));
    end 
  end

  % Read in the indices for the train/test partitions
  partitionFile = fullfile(dataDir, [dataDir '-Testpct=20-nTests=20-nFold=10.idx']);
  [testIdxMat, crossValIdxCell, nTests, nFolds] = readPartitionFile(partitionFile);

  % File where to store results
  [notUsed, resultFileTmp, fileExt] = fileparts(partitionFile);
  resultFile = ['result_regLambda_' timeStamp '_' resultFileTmp '.mat'];
  resultFile = fullfile(dataDir, resultFile);

  % Do we assume we know the similarity domain or not?
  % Pick one and stick with it.
  %simDomain = unique(simMat(:));

  simDomain = determineSimDomain(dataDir, simMat);

  % Parameters for each data set
  nSamples = length(classLabels);
  nTestSamplesAll = size(testIdxMat,2);
  nCrossValSamplesAll = size(crossValIdxCell{1},2);
  nTrainSamplesAll = nSamples - nTestSamplesAll - nCrossValSamplesAll;

  % Define the neighborhood sizes to test for local classifiers
  maxPwr2 = floor(log2(nTrainSamplesAll));
  neighborhoodSize= 2.^(1:maxPwr2);
  neighborhoodSize = unique([1:16 neighborhoodSize]);
  % Limit size to 128
  maxSize = 128;
  neighborhoodSize(neighborhoodSize > maxSize) = [];
   
  % FOR DEBUGGING OLY - COMMENT OUT FOR REAL RUN
  %nTests = 1;
  %END DEBUGGING

  neighborType = 'kTotal'
  lambdaType = 'global'
  
  if strcmp(neighborType, 'kPerClass')
    % Adjust neighborhood sizes to be per class
    maxNeighborhoodSize = determineMaxNeighborhoodSizeByClass(classLabels, ...
      crossValIdxCell, testIdxMat);
    thisNeighborhoodSize = [neighborhoodSize(:); maxNeighborhoodSize];
    thisNeighborhoodSize(thisNeighborhoodSize > maxNeighborhoodSize) = [];
    thisNeighborhoodSize = unique(thisNeighborhoodSize)
  else
    thisNeighborhoodSize = neighborhoodSize
  end
  
  %% FOR DEBUGGING
  %thisNeighborhoodSize = [1:2];
  %thisNeighborhoodSize = 1:2;
  %thisNeighborhoodSize = 1:5;
  %% END DEBUGGING

  bestSize = zeros(nTests,1);
  bestRegParam = -1e6 * ones(nTests,1);
  minErr = 1e6*ones(nTests,1);
  errCount = -1e6 * ones(nTests,1);
  nonRegModelParams = cell(nTests,1);
  posteriors = cell(nTests,1);
  
  regParamVec = [1e-6 1e-3 0.01 0.1 0.5 0.9]
  %regParamVec = [1e-6 0.1];
  %regParamVec = 1e-6;
  %regParamVec = 0

  for iTest = 1:nTests

    disp(['Test split ' num2str(iTest) ' ...']);

    % Cross-validate number of neighbors

     crossValIdxMat = crossValIdxCell{iTest};
     [bestSize(iTest), bestRegParam(iTest),  minErr(iTest)] = ...
       crossValidateParamsRegularizedLambdaLocalSda( thisNeighborhoodSize, ...
       neighborType, regParamVec, simMat, crossValIdxMat, classLabels, ...
       simDomain, testIdxMat(iTest,:), lambdaType);
     
    % Test using cross-validated parameters

    testIdx = testIdxMat(iTest,:);
    trainSimMat = simMat;
    trainSimMat(:, testIdx) = [];
    trainSimMat(testIdx,:) = [];
    trainClassLabels = classLabels;
    trainClassLabels(testIdx) = [];
    
    % Assume default is 'average' lambda regularization
    regularizingParams = createRegularizedLambdaParamStruct( ...
      'average', [], bestRegParam(iTest), neighborType, 'maxSumSim', ...
      bestSize(iTest), simDomain, []);
      
    % ... but we could use 'global' ...
    if strcmpi(lambdaType, 'global');
      regularizingParams.lambdaType = 'global';
      modelParams = computeSdaModels(trainSimMat, trainClassLabels, ...
        'maxSumSim', simDomain);
      regularizingParams.regLambda = modelParams.lambda;
      regularizingParams.classLabels = modelParams.classLabels;
    end
    
    [errCount(iTest), notUsed, posteriors{iTest}, nonRegModelParams{iTest}] = ...
      evaluateTestsRegularizedLambdaLocalSda(regularizingParams, simMat, testIdx, ...
      classLabels);
    
    nTestSamples(iTest) = length(testIdx);

    disp(['Errors in this test split: ' num2str(errCount(iTest)) '/' num2str(nTestSamples(iTest))]); 
    
    save(resultFile)
    
  end % for iTest
  errPct = errCount(:) ./ nTestSamples(:);
  avgErrPct = mean(errPct)
  stdErrPct = std(errPct, 1)
  
  save(resultFile)

end % for iDir
