// Implementation file for computing the SDA parameters. Uses the Brent's method for function
// minimization. This implementation of brent() is due to the authors below and
// is covered by the license below:
// GNU General Public License Agreement
// Copyright (C) 2004-2007 CodeCogs, Zyba Ltd, Broadwood, Holford, TA5 1DU, England.
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by CodeCogs. 
// You must retain a copy of this licence in all copies. 
//
// This program is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
// PARTICULAR PURPOSE. See the GNU General Public License for more details.
// ---------------------------------------------------------------------------------

// All other functions were originally written by Ingrid Liu and Peter Sadowski,
// Information Design Lab, University of Washington, Summer/Fall 2009. Adapted by Luca
// Cazzanti, Applied Physics Lab, Univ. Washington to be use as a mex file, Fall 2009.
// First complete version: Dec 1, 2009.  


#include "BrentExp.h"

#define INF 1.7E308
#define RGOLD       0.3819660
#define ZEPS        1.0E-10
#define MINH(x, y)   ((x) < (y) ? (x) : (y))
#define MAXH(x, y)   ((x) > (y) ? (x) : (y))
#define ABS(x)      ((x) < 0 ? -(x) : (x))
#define SIGN(x, y)  ((y) < 0 ? -ABS(x) : ABS(x))

namespace idl
{
  
  bool getLambdaGamma(const RealMatrix &domainVector, double classMean, double &lambda, double &gamma) 
  {
    double maxSim = domainVector.max();
    double minSim = domainVector.min();
    if (classMean == maxSim) {
      // Degenerate case where label G samples are of max similarity to the centroid H.
      // This could include simDomain of size 1.
      lambda = -1 * INF;
      gamma = 1;
    } else if(classMean == minSim) {
      // Degenerate case where label G samples are of min similarity to the centroid H.
      lambda = (-1) * INF;
      gamma = 1;
    } else {
      // Find optimal lambda for local discrete exponential.
      lambda = brent(&idl::meanConstraintFxn, domainVector, classMean, -100, 0, 100, 1.0E-8, 100);
      double theErr = meanConstraintFxn(lambda, domainVector, classMean);
      //if (theErr > .01) {
      //cout << "Warning: Error large. Possibly because sim domain is large." << endl; }
      
      RealMatrix elambdaVect = lambda*domainVector;
      elambdaVect.fxn(exp);
      double sumELambdaVect = elambdaVect.sum();
      if (fabs(sumELambdaVect) < ZEPS) {
	// PMF is numerically sensitive.
	lambda = (-1) * INF;
	gamma = INF;
      } else {
	gamma = 1/sumELambdaVect;
      }
    }
    return true;
  }

  //! Calculates the minimum of a one-dimensional real function using Brent's method.
  
  double brent (double (*f)(double,const RealMatrix&,double), const RealMatrix &domainVector, double classMean, double a, double b, double c,
		double eps, int maxit)
  {
    
    double ax = MINH(a, c), bx = MAXH(a, c), u, v = b,
      w = v, x = v, d = 0, e = 0, fx = f(x,domainVector,classMean), fv = fx, fw = fx;
    
    for (int i = 0; i < maxit; i++) {
      
      bool flag = true;
      double xm = (ax + bx) / 2, eps1 = eps * ABS(x) + ZEPS, eps2 = 2 * eps1;
      
      if (ABS(x - xm) <= eps2 - (bx - ax) / 2) break;
      
      if (ABS(e) > eps1) {
	
	double r = (x - w) * (fx - fv), q = (x - v) * (fx - fw),
	  p = (x - v) * q - (x - w) * r, etemp = e;
	
	q = (q - r) / 5;
	if (q > 0) p *= -1;
	q = ABS(q), e = d;
	
	if (ABS(p) < ABS(q * etemp / 2) && p > q * (ax - x) && p < q * (bx - x)) {
	  flag = false, d = p / q, u = x + d;
	  if (u - ax < eps2 || bx - u < eps2) d = SIGN(eps1, xm - x);
	}
	
      }
      
      if (flag) (e = ((x < xm) ? bx : ax) - x);
      
      d = RGOLD * e;
      u = x + (ABS(d) < eps1 ? SIGN(eps1, d) : d);
      
      double fu = f(u,domainVector,classMean);
      if (fu <= fx) {
	
	(u < x ? bx : ax) = x;
	v = w, fv = fw;
	w = x, fw = fx;
	x = u, fx = fu;
	
      }
      else {
	
	(u < x ? ax : bx) = u;
	
	if (fu <= fw || w == x)
	  v = w, fv = fw, w = u, fw = fu;
	else
	  if (fu <= fv || v == x || v == w)
	    v = u, fv = fu;
	
      }
    }
    
    return x;
    
  }

  double meanConstraintFxn(double lambda, const RealMatrix &domainVector, double classMean)
  {
    RealMatrix elambdaVect = lambda*domainVector;
    elambdaVect.fxn(exp);
    
    double sumELambdaVect = elambdaVect.sum();
    double theErr = 0.0;
    if(sumELambdaVect < ZEPS)
      {
	//cerr << "Something weird has happened in the meanConstraintFxn";
	lambda = (-1)*INF; 
	return theErr;
      }
    // normalize the pmf 
    elambdaVect /= sumELambdaVect;
    
    //and then calc the mean:
    double meanpmf = (elambdaVect.dot(domainVector)).sum();
    
    // absolute error to empirical mean
    theErr = fabs(meanpmf - classMean); 
    return theErr;
  }
  
  
  
}

#undef SIGN
#undef ABS
#undef MAXH
#undef MINH
#undef ZEPS
#undef RGOLD

//#endif

