// File: Matrix.h
// Purpose: Definition of the Matrix class template
// Author: Yihua Chen (yhchen@u.washington.edu)
// Last modified: 1/30/2008 5:50:26 PM

#ifndef IDL_MATRIX_H
#define IDL_MATRIX_H

#include <vector>
#include <algorithm>

namespace idl
{
	/** Class template for matrix. */
	template <class T>
	class Matrix
	{
	public:
		/** Default constructor. */
		Matrix() : nRows(0), nColumns(0) {}

		/**
		 * Constructor.
		 * @param[in]	m	row number
		 * @param[in]	n	column number
		 */
		Matrix(unsigned int m, unsigned int n)
			: nRows(m), nColumns(n), elements(m * n)
		{
		}

		/**
		 * Constructor.
		 * @param[in]	m		row number
		 * @param[in]	n		column number
		 * @param[in]	initVal	initial value for all the elements
		 */
		Matrix(unsigned int m, unsigned int n, const T &initVal)
			: nRows(m), nColumns(n), elements(m * n, initVal)
		{
		}

		/**
		 * Copy constructor.
		 * @param[in]	mat	matrix to copy from
		 */
		Matrix(const Matrix<T> &mat)
			: nRows(mat.nRows), nColumns(mat.nColumns), elements(mat.elements)
		{
		}

		virtual ~Matrix() {} /**< Destructor. */

		/** Erase the elements of the matrix. */
		void clear()
		{
			elements.clear();

			nRows = 0;
			nColumns = 0;
		}

		/**
		 * Specify a new size for the matrix.
		 * @param[in]	m	row number
		 * @param[in]	n	column number
		 */
		void resize(unsigned int m, unsigned int n)
		{
			elements.resize(m * n);

			nRows = m;
			nColumns = n;
		}

		/**
		 * Dump all the elements to an STL vector.
		 * @param[out]	vec		vector to which the elements are dumped
		 */
		void dump(std::vector<T> &vec)
		{
			vec.assign(elements.begin(), elements.end());
		}

		/**
		 * Get the number of rows.
		 * @return	number of rows
		 */
		unsigned int getRowNum() const
		{
			return nRows;
		}

		/**
		 * Get the number of columns.
		 * @return	number of columns
		 */
		unsigned int getColumnNum() const
		{
			return nColumns;
		}

		/**
		 * Get the maximum of the row and column number.
		 * @return	maximum of the row and column number
		 */
		unsigned int length() const
		{
			return (nRows > nColumns) ? nRows : nColumns;
		}

		/**
		 * Get the size of the matrix.
		 * @return	size of the matrix
		 */
		std::vector<unsigned int> size() const
		{
			std::vector<unsigned int> dims(2);
			dims[0] = nRows;
			dims[1] = nColumns;

			return dims;
		}

		/**
		 * Test if the matrix is empty.
		 * @return	true if empty; false if not empty
		 */
		bool isEmpty() const
		{
			return (0 == nRows || 0 == nColumns);
		}

		/**
		 * Test if the matrix is square.
		 * @return	true if square; false if not square
		 */
		bool isSquare() const
		{
			return (nRows == nColumns);
		}

		/**
		 * Test if the matrix is a vector.
		 * @return	true if the matrix is a vector; false if the matrix is not a vector
		 */
		bool isVector() const
		{
			return ((nRows == 1 && nColumns > 0) || (nColumns == 1 && nRows > 0));
		}

		/**
		 * Get the transpose of the matrix.
		 * @return	transpose of the matrix
		 */
		Matrix<T> transpose() const
		{
			Matrix<T> A(nColumns, nRows);

			for (unsigned int i = 0; i < nRows; ++i)
			{
				for (unsigned int j = 0; j < nColumns; ++j)
				{
					A(j, i) = elements[i * nColumns + j];
				}
			}

			return A;
		}

		/**
		 * Get a row of the matrix.
		 * @param[in]	i	row index
		 * @return		row vector
		 */
		Matrix<T> row(unsigned int i) const
		{
			Matrix<T> xi(1, nColumns);

			for (unsigned int j = 0; j < nColumns; ++j)
			{
				xi(j) = elements[i * nColumns + j];
			}

			return xi;
		}

		/**
		 * Get rows of the matrix.
		 * @param[in]	i1	starting row index
		 * @param[in]	i2	ending row index
		 * @return		block matrix
		 */
		Matrix<T> rows(unsigned int i1, unsigned int i2) const
		{
			unsigned int m = (i1 < i2) ? (i2 - i1 + 1) : (i1 - i2 + 1);
			unsigned int k = i1;

			Matrix<T> A(m, nColumns);

			for (unsigned int i = 0; i < m; ++i)
			{
				for (unsigned int j = 0; j < nColumns; ++j)
				{
					A(i, j) = elements[k * nColumns + j];
				}
				if (i1 < i2) ++k; else --k;
			}

			return A;
		}

		/**
		 * Get rows of the matrix.
		 * @param[in]	is	row indices
		 * @return		block matrix
		 */
		Matrix<T> rows(const std::vector<unsigned int> &is) const
		{
			unsigned int m = static_cast<unsigned int>(is.size());

			Matrix<T> A(m, nColumns);

			for (unsigned int i = 0; i < m; ++i)
			{
				for (unsigned int j = 0; j < nColumns; ++j)
				{
					A(i, j) = elements[is[i] * nColumns + j];
				}
			}

			return A;
		}

		/**
		 * Get a column of the matrix.
		 * @param[in]	j	column index
		 * @return		column vector
		 */
		Matrix<T> column(unsigned int j) const
		{
			Matrix<T> xj(nRows, 1);

			for (unsigned int i = 0; i < nRows; ++i)
			{
				xj(i) = elements[i * nColumns + j];
			}

			return xj;
		}

		/**
		 * Get columns of the matrix.
		 * @param[in]	j1	starting column index
		 * @param[in]	j2	ending column index
		 * @return		block matrix
		 */
		Matrix<T> columns(unsigned int j1, unsigned int j2) const
		{
			unsigned int n = (j1 < j2) ? (j2 - j1 + 1) : (j1 - j2 + 1);
			unsigned int k = j1;

			Matrix<T> A(nRows, n);

			for (unsigned int i = 0; i < nRows; ++i)
			{
				for (unsigned int j = 0; j < n; ++j)
				{
					A(i, j) = elements[i * nColumns + k];
				}
				if (j1 < j2) ++k; else --k;
			}

			return A;
		}

		/**
		 * Get columns of the matrix.
		 * @param[in]	js	column indices
		 * @return		block matrix
		 */
		Matrix<T> columns(const std::vector<unsigned int> &js) const
		{
			unsigned int n = static_cast<unsigned int>(js.size());

			Matrix<T> A(nRows, n);

			for (unsigned int i = 0; i < nRows; ++i)
			{
				for (unsigned int j = 0; j < n; ++j)
				{
					A(i, j) = elements[i * nColumns + js[j]];
				}
			}

			return A;
		}

		//@{
		/**
		 * Get a block of the matrix.
		 * @param[in]	i/is	row index/indices
		 * @param[in]	j/js	column index/indices
		 * @return		block matrix
		 */
		Matrix<T> operator () (unsigned int i, const std::vector<unsigned int> &js) const
		{
			unsigned int n = static_cast<unsigned int>(js.size());

			Matrix<T> xi(1, n);

			for (unsigned int k = 0; k < n; ++k)
			{
				xi(k) = elements[i * nColumns + js[k]];
			}

			return xi;
		}
		Matrix<T> operator () (const std::vector<unsigned int> &is, unsigned int j) const
		{
			unsigned int m = static_cast<unsigned int>(is.size());

			Matrix<T> xj(m, 1);

			for (unsigned int k = 0; k < m; ++k)
			{
				xj(k) = elements[is[k] * nColumns + j];
			}

			return xj;
		}
		Matrix<T> operator () (const std::vector<unsigned int> &is,
							   const std::vector<unsigned int> &js) const
		{
			unsigned int m = static_cast<unsigned int>(is.size());
			unsigned int n = static_cast<unsigned int>(js.size());

			Matrix<T> A(m, n);

			for (unsigned int i = 0; i < m; ++i)
			{
				for (unsigned int j = 0; j < n; ++j)
				{
					A(i, j) = elements[is[i] * nColumns + js[j]];
				}
			}

			return A;
		}
		//@}

		//@{
		/**
		 * Get the element in the ith row and jth column.
		 * @param[in]	i	row index
		 * @param[in]	j	column index
		 * @return		reference to the element
		 */
		T & operator () (unsigned int i, unsigned int j)
		{
			return elements[i * nColumns + j];
		}
		const T & operator () (unsigned int i, unsigned int j) const
		{
			return elements[i * nColumns + j];
		}
		//@}

		//@{
		/**
		 * Get the ith element.
		 * @param[in]	i	element index
		 * @return		reference to the element
		 */
		T & operator () (unsigned int i)
		{
			return elements[i];
		}
		const T & operator () (unsigned int i) const
		{
			return elements[i];
		}
		//@}

		/**
		 * Assignment operator.
		 * Copy the contents of another matrix.
		 * @param[in]	mat matrix to copy from
		 * @return		reference to itself
		 */
		Matrix<T> & operator = (const Matrix<T> &mat)
		{
			nRows = mat.nRows;
			nColumns = mat.nColumns;
			elements = mat.elements;

			return *this;
		}

		/**
		 * Assign a value to all the elements.
		 * @param[in]	val	value to be assigned
		 * @return		reference to itself
		 */
		Matrix<T> & fill(const T &val)
		{
			std::fill(elements.begin(), elements.end(), val);

			return *this;
		}

		/**
		 * Assign a value to all the elements in a row.
		 * @param[in]	i	row index
		 * @param[in]	val	value to be assigned
		 * @return		reference to itself
		 */
		Matrix<T> & fillRow(unsigned int i, const T &val)
		{
			std::fill_n(elements.begin() + i * nColumns, nColumns, val);

			return *this;
		}

		/**
		 * Assign a value to all the elements in a column.
		 * @param[in]	j	column index
		 * @param[in]	val	value to be assigned
		 * @return		reference to itself
		 */
		Matrix<T> & fillColumn(unsigned int j, const T &val)
		{
			unsigned int b = nRows * nColumns;
			
			for (unsigned int k = j; k < b; k += nColumns)
			{
				elements[k] = val;
			}

			return *this;
		}

	protected:
		unsigned int nRows;		/**< Number of rows. */
		unsigned int nColumns;	/**< Number of columns. */

		std::vector<T> elements; /**< Container of the elements. */
	};

	/** Type definition for integer matrix. */
	typedef Matrix<int> IntMatrix;

	/** Type definition for unsigned integer matrix.
	 *  Redefined by Peter in UIntMatrix.cpp*/
	//typedef Matrix<unsigned int> UIntMatrix;
}

#endif // IDL_MATRIX_H
