function indexMat = adjustSelfSimilarityRank(indexMat)
%adjustSelfSimilarityRank - force self-similarity to have rank 1.
% Adjust the rank of a sample's nearest neighbors so that the sample is
% always its own nearest neighbor. This function is suited for use in
% conjunction with findNearNeighbors(). The latter function uses
% Matlab's sort() to sort a sample's nearest neighbors. In case of ties,
% sort() orders the results by keeping the original order of the  tied
% neighbors. This is not always desirable, as we often assume that a
% sample's nearest neighbor is the sample itself.
%
% USAGE: adjustedIndexMat = adjustSelfSimilarityRank(indexMat)
% 
% EXAMPLE: The pairwise similarity of three samples is below. Finding the
% indices of each sample's the closest neighbors produces the wrong order
% for the second sample. The order is fixed by adjustSelfSimilarityRank.
% simVec = [10 10 5;
%           10 10 3;
%           5  3 10];
% neighborIdx = findNearNeighbors(simVec)
% neighborIdx =
%     1     2     3
%     1     2     3
%     3     1     2
% adjustNeighborIdx = adjustSelfSimilarityRank(neighborIdx)
% adjustNeighborIdx =
%     1     2     3
%     2     1     3
%     3     1     2
%
% See also findNearNeighbrs.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, Jan 2007.
%$Id:$
%$DateTime:$
%$Author:$

% Assumption: indexMat is square and max(max(indexMat)) < size(indexMat)
[nData, notUsed] = size(indexMat); %#ok
correctRowIdx = 1:nData;
correctRowIdx = correctRowIdx.';
rowsToChange = find( indexMat(:,1) ~= correctRowIdx );
rowsToChange = rowsToChange.';
for rowIdx = rowsToChange
  colIdx = find( indexMat(rowIdx, :) == rowIdx );
  tmpVec = indexMat(rowIdx, 1:colIdx);
  tmpVec = circshift(tmpVec, [0 1]);
  indexMat(rowIdx, 1:colIdx) = tmpVec;
end %for rowIdx

return