function [lambda, gamma] = applyMaxEntParameterCorrection(lambda, gamma, ...
  meanSim, simToCentroidsVec)
%applyMaxEntParameterCorrection - handle case of lambda = -Inf.
% Corrects the value of the exponent in the maximum entropy models when the
% value returned by computeMaxEntModels() is -Inf. Any elements of
% lambda equal to -Inf are set to 0 and the corresponding gamma are set to
% 1, if the corresponding input similarities are equal to the mean
% similarity constraint used to compute lambda and gamma.
% 
% USAGE
%[lambdaOut, gammaOut] = applyMaxEntParameterCorrection(lambda, gamma, ...
%     meanSim, simToCentroids)
% lambda: C-by-C matrix of exponents as returned by computeMaxEntModels(),
%     where C is the number of classes.
% gamma: C-by-C matrix of scaling factors as returned by computeMaxEntModels().
% meanSim: matrix of mean similarity constraints as returned by
%     computeMaxEntModels().
% simToCentroids: 1-by-C vector of similarities to the centroids of each class 
%
% See also computeMaxEntModels().

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, Jan 2009.
%$Id:$
%$DateTime:$
%$Author:$

% How a "degenerate" similarity distribution is handled:
% Check if lambda == -Inf: if yes, then the pmf is a discrete delta
% function centered at meanSim. Manually set
% P(sim(x, mu_h) = meanSim_gh | Y=g ) = 1 if the test similarity is equal
% to the mean similarity. The class-conditional probability of all other
% similarity values are automatically set to 0 due to -Inf in exponent.

infiniteLambdaIdx = lambda == -Inf; %logical matrix
if any(infiniteLambdaIdx(:)) == true
  simMat = repmat(simToCentroidsVec(:).', size(gamma,1), 1);
  meanSimIdx = meanSim == simMat; 
  degenerateIdx = infiniteLambdaIdx & meanSimIdx;
  lambda(degenerateIdx) = 0; %#ok, log(1) = 0; we use log-discriminant
  gamma(degenerateIdx) = 1;
end

return