function [classEst, logPosteriors, logMarginals, modelParams] = ...
  classifyClassMtmrPairwiseLocalSda( testPointSim, neighborSimMat, ...
  neighborClassLabels, sigma, eta, simDomain)
%classifyClassMtmrPairwiseLocalSda - classify a test point with MTMR local SDA.
% Classifies a test point based on its similarity to a set of other points
% using local similarity discriminant analysis (SDA) and multi-task
% manifold regularization (MTMR), where the regularization is across the
% possible classes.
%
% USAGE
% TO DO - THIS IS COPIED FROM LOCAL SDA
% [classEst, logPosteriors, modelParams, ...
%    sameClassLogPosteriors, sameClassEst, logMarginals] = classifyLocalSda( ...
%    simVec, simMat, classLabels, centroidType, simDomain)
% simVec: 1-by-M vector of test point similarities to each of the set points.
% simMat: M-by-M matrix of pairwise similarities of all set points
% classLabels: M-by-1 vector of class labels of the points in the set.
% centroidType - string denoting the type of centroid to use. See
%     findCentroid() for a list of allowed centroid types.
% simDomain - vector of all possible similarity values. Optional parameter;
%     if it is not provided, it is constructed from simMat.
% classEst - class estimate for the test point corresponding to simVec.
% logPosteriors - log of the posterior probabilites for the classes.
% modelParams - structure with the computed maximum entropy model
%     parameters used for classification. See computeMaxEntModels for
%     details.
% sameClassLogPosteriors - disabled, returns -Inf.
% sameClassEst - disabled, returns 0.
% logMarginals - G-by-G matrix of the log of the class-conditional marginal
%     probabilities, where G is the number of classes. The output
%     logPosteriors is computed from this matrix.
%
% See also classifySda

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of washington, August 2010.
%$Id:$
%$DateTime:$
%$Author:$

% Initialize some variables 
uniqueClassLabels = unique(neighborClassLabels);
nClasses = length(uniqueClassLabels);

%% All neighbors are from one class --> classify as that class; done.
if nClasses == 1
  classEst = uniqueClassLabels;
  logPosteriors = 0;
  logMarginals = 0;
  modelParams = initSdaModels; %creates empty/default structure
  modelParams.classPriors = 1;
  modelParams.classCounts = size(neighborSimMat,1);
  modelParams.classLabels = uniqueClassLabels;
  % The other fields are empty
  
  return
end


%% All other cases
% Classify based on regularizing the means across classes, then computing
% the local SDA parameters from the regularized means. This is the core of
% this flavor of SDA-MTMR.

% EXPERIMENTAL - value of sigma based only on local sims
%localSimDomain = unique([testPointSim(:); neighborSimMat(:)]);
%sigmaOld = sigma;
%maxLocalSim = max([testPointSim(:); neighborSimMat(:)]);
%minLocalSim = min([testPointSim(:); neighborSimMat(:)]);
%sigma = (0.4 * (maxLocalSim - minLocalSim))^2;
%if sigma < eps
%  sigma = sigmaOld;
%end
% END EXPERIMENTAL

modelParams = computeClassMtmrPairwiseSimParams(neighborSimMat, ...
  neighborClassLabels, simDomain, sigma);
maxSim = max(simDomain);
minSim = min(simDomain);

% Vectorize for later - these are used inside all loops
classCountsMat = repmat(modelParams.classCounts(:), 1, nClasses);
sumSims = modelParams.meanSim .* classCountsMat;
classCountsMat = classCountsMat(:); 
sumSims = sumSims(:);

nSigmas = length(sigma);
nEtas = length(eta);

nIters = 1; % experimental, for now keep this equal to 1
classEst = -1000 * ones(nEtas, nSigmas);
for iSigma = 1:nSigmas
  
  thisClassSim = modelParams.classSim(:,:,iSigma);
  
  classEstTemp = -1000 * ones(nEtas, 1);
  for iEta = 1:nEtas
    
    newMeans = -1000 * ones(nClasses^2);
    thisLambdaVec = -1000 * ones(nClasses^2,1);
    thisGammaVec =  -1000 * ones(nClasses^2,1);
    thisEstErrVec = -1000 * ones(nClasses^2,1);
    
    iterClassSim = thisClassSim;
    for iIter = 1:nIters
      
      classSimDiag = diag(iterClassSim);
      denominator = classCountsMat + ...
        eta(iEta) * (sum(iterClassSim,2) - classSimDiag );

      tildeParam = sumSims ./ denominator;
      normSimMat = eta(iEta) * iterClassSim ./ ...
        repmat(denominator, 1, nClasses^2);
      for iDim = 1:nClasses^2
        normSimMat(iDim, iDim) = 0;
      end

      newMeans = ( eye(nClasses^2) - normSimMat ) \ tildeParam;
      newMeans = reshape(newMeans, nClasses, nClasses);
      iterClassSim  = computeRbfSimClassMtmr(newMeans, sigma(iSigma));
      iterClassSim = squeeze(iterClassSim);
      
    end % for iIter
    
    
    %  ... compute local SDA parameters from regularized means:
    % pick either Mex or m-script implementation ...
    
    %  ... Mex is faster but tested only on Linux and Mac ...
    nSims = length(simDomain);
    for iMean = 1:nClasses^2
      if (abs(newMeans(iMean) - maxSim) < eps) || ...
          (abs(newMeans(iMean) - minSim) < eps)
        thisLambdaVec(iMean) = -Inf;
        thisGammaVec(iMean) = 1; % either case
        thisEstErrVec(iMean)= 0; % either case
      else
        %  ... Mex is faster but tested only on Linux ...
        [thisLambdaVec(iMean), thisGammaVec(iMean), ...
          thisEstErrVec(iMean)] = MexComputeLambda( ...
          newMeans(iMean), simDomain, nSims);
         
        % ... M-file is slower but more portable ...
        %for iMean = 1:nClasses^2;
        %[thisLambdaVec(iMean), thisGammaVec(iMean), ...
        %  thisEstErrVec(iMean)] = computeLambda(newMeans(iMean), simDomain);
        %end %for iMean
      end % if-then-else

    end % for iMean
    
    % ... classify ...
    modelParams.lambda = reshape(thisLambdaVec, nClasses, nClasses);
    modelParams.gamma = reshape(thisGammaVec, nClasses, nClasses);
    modelParams.meanSim = newMeans;
    logMarginals = 1e6 * ones(nClasses);
    for gIdx = 1:nClasses
      for hIdx = 1:nClasses
        hClassNeighborIdx = neighborClassLabels == uniqueClassLabels(hIdx);
        testSimsToClassH = testPointSim(hClassNeighborIdx);
        thisLambda = modelParams.lambda(gIdx,hIdx);
        thisGamma = modelParams.gamma(gIdx,hIdx);
        thisMeanSim = modelParams.meanSim(gIdx,hIdx);
        thisExponent = thisLambda * testSimsToClassH;
        % CAREFUL: if thisLambda is -Inf and a testSimToClassH contains a 0,
        % the corresponding elements in the product are -NaN. Instead we want
        % to keep the -Inf, so we fix this here.
        % LGC, Sept. 2010 
        nanProblemIdx = isnan(thisExponent);
        thisExponent(nanProblemIdx) = -Inf;
        thisProb = thisGamma * exp(thisExponent);
        % CAREFUL AGAIN: any degenerate pmfs (Kronecker delta) must be
        % re-introduced here. In computePairwiseMaxEntModels() we manually set
        % lambda = -Inf and gamma = 1 to handle the degenerate pmf case, which
        % happens if meanSim = max (or min) sim. We check for lambda = -Inf,
        % which sets all probabilities to 0 and we re-introduce a probability = 1
        % for the extremal similarity value. This is consistent with
        % applyMaxEntParameterCorrection(), which does the same thing for
        % sim-to-centroid similarities and with classifyPairwiseLocalSda().
        %LGC, Sept. 2010
        if isinf(thisLambda) && ( ( abs(thisMeanSim - maxSim) < eps ) || ...
            ( abs(thisMeanSim - minSim) < eps )  )
          setProbToOneIdx = abs(testSimsToClassH - thisMeanSim) < eps;
          thisProb(setProbToOneIdx) = 1; % log(1) = 0;
        end
        logMarginals(gIdx, hIdx) = log(mean(thisProb));
      end %for hIdx
    end %for gIdx
    
    logPosteriors = sum(logMarginals,2) + log(modelParams.classPriors(:));
    
    % Do all classes have 0 probability? 
    if all( isinf(logPosteriors) )
      % THIS IS FOR DEBUGGING - using nearest centroid is consistent with
      % classifyLocalSda(), but it is inconsistent with using all pairwise
      % similarities. This code is for debugging ...
      %centroidParams = computeCentroidModels(neighborSimMat, ...
      %  neighborClassLabels, 'maxSumSim');
      %classEst = classifyNearestCentroid(testPointSim, centroidParams, 'similarity');
      % END DEBUG
  
      % DO THIS INSTEAD: classify as the class whose mean pairwise similarity
      % to the test point is maximum - This is comsistent with
      % classifyPairwiseLocalSda()
      classEstTemp(iEta) = classifyMaximumMeanSimilarity(testPointSim, neighborClassLabels);
      
      % NOTE: should we use the new mean pairwise sims?
      % The code below replicates classifyMaximumMeanSimilarity(), but acts
      % on the regularized mean pariwise sims instead of the original ones.
      % LGC - tired it but classification is slightly worse.
      
      %meanSimVec = diag(newMeans);
      %[sortedSimVec, sortIdx] = sort(meanSimVec);
      %sortedClassLabels = modelParams.classLabels(sortIdx);
      %sortedClassPriors = modelParams.classPriors(sortIdx);
      
      % Are there any ties for maximum sim? Break them according to class prior
      %tieIdx = sortedSimVec == sortedSimVec(end); % assumes similarity so max is last element
      %tiePriors = sortedClassPriors(tieIdx);
      %tieLabels = sortedClassLabels(tieIdx);
      %[notUsed, maxIdx] = max(tiePriors); %#ok
      %classEstTemp(iEta) = tieLabels(maxIdx);
      
    else
      [notUsed, classEstIdx] = max(logPosteriors); %#ok
      classEstTemp(iEta) = modelParams.classLabels(classEstIdx);
    end
    
  end %for iEta
  
  classEst(:, iSigma) = classEstTemp(:);
  
end % for iSigma

return %END main function
