function [classEst, logPosteriors, modelParams, sameClassLogPosteriors ...
  sameClassEst, logMarginals] = classifyMtmrLocalSda(...
  testPointSim, neighborSimMat, neighborClassLabels, centroidType, ...
  sigma, eta, varargin)
%classifyLocalSda - classify a test point with MTMR local SDA.
% Classifies a test point based on its similarity to a set of other points
% using local similarity discriminant analysis (SDA) and multi-task
% manifold regularization (MTMR)
%
% USAGE
% TO DO - THIS IS COPIED FROM LOCAL SDA
% [classEst, logPosteriors, modelParams, ...
%    sameClassLogPosteriors, sameClassEst, logMarginals] = classifyLocalSda( ...
%    simVec, simMat, classLabels, centroidType, simDomain)
% simVec: 1-by-M vector of test point similarities to each of the set points.
% simMat: M-by-M matrix of pairwise similarities of all set points
% classLabels: M-by-1 vector of class labels of the points in the set.
% centroidType - string denoting the type of centroid to use. See
%     findCentroid() for a list of allowed centroid types.
% simDomain - vector of all possible similarity values. Optional parameter;
%     if it is not provided, it is constructed from simMat.
% classEst - class estimate for the test point corresponding to simVec.
% logPosteriors - log of the posterior probabilites for the classes.
% modelParams - structure with the computed maximum entropy model
%     parameters used for classification. See computeMaxEntModels for
%     details.
% sameClassLogPosteriors - disabled, returns -Inf.
% sameClassEst - disabled, returns 0.
% logMarginals - G-by-G matrix of the log of the class-conditional marginal
%     probabilities, where G is the number of classes. The output
%     logPosteriors is computed from this matrix.
%
% See also classifySda

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of washington, November 2006.
%$Id:$
%$DateTime:$
%$Author:$

uniqueClassLabels = unique(neighborClassLabels);
nClasses = length(uniqueClassLabels);
logPosteriors = -Inf * ones(1, nClasses);
modelParams = initSdaModels;
modelParams.classLabels = uniqueClassLabels;

sameClassLogPosteriors = -Inf * ones(1, nClasses);

%% All neighbors are from one class --> classify as that class; done.
if nClasses == 1
  classEst = uniqueClassLabels;
  logPosteriors = 0;
  logMarginals = 0;
  modelParams.classPriors = 1;
  modelParams.classCounts = size(neighborSimMat,1);
  
  sameClassLogPosteriors = 0;
  sameClassEst = uniqueClassLabels;
  %The other fields are empty
  return
end

%% A class has only one neighbor --> classify using nearest-centroid; done.
% It won't be necessary to check for this special case --> folded into all
% other cases
if(0)
nSamplesPerClass = hist(neighborClassLabels, uniqueClassLabels);
if any(nSamplesPerClass == 1)
  [classEst, modelParams] = classifyLocalNearestCentroid( ...
    testPointSim, neighborSimMat, neighborClassLabels, centroidType);
  sameClassEst = classEst;
  
  % For consistency, add these empty fields
  modelParams.gamma = [];
  modelParams.lambda = [];
  modelParams.meanSim = [];
  modelParams.estErr = [];
  modelParams.simDomain = [];
  logMarginals = 0;
  return

end % if any(nSamplesPerClass == 1)
end % if(0)

%% All other cases --> classify based on MaxEnt models
simDomain = local_determineSimDomain(testPointSim, neighborSimMat, ...
  neighborClassLabels, centroidType, varargin{:});

modelParams = computeMtmrSimParams(neighborSimMat, neighborClassLabels, ...
  centroidType, simDomain, sigma);

%  ... regularize the means - this is the core of this approach
classCountsMat = repmat(modelParams.classCounts(:), 1, nClasses);
sumSimToCentroids = modelParams.meanSim .* classCountsMat;

% Vectorize for later
%classCountsMat = classCountsMat.';
classCountsMat = classCountsMat(:); 
%simSimToCentroids = sumSimToCentroids.';
sumSimToCentroids = sumSimToCentroids(:);

classSimDiag = diag(modelParams.classSim);
denominator = classCountsMat + ...
  eta * (sum(modelParams.classSim,2) - classSimDiag ); 

tildeParam = sumSimToCentroids ./ denominator;
normSimMat = eta * modelParams.classSim ./ repmat(denominator, 1, nClasses^2);
for iDim = 1:nClasses^2
  normSimMat(iDim, iDim) = 0;
end

newMeans = ( eye(nClasses^2) - normSimMat ) \ tildeParam;


%  ... compute local SDA parameters from regularized means

modelParams.lambda = -1000 * ones(nClasses^2,1);
modelParams.gamma =  -1000 * ones(nClasses^2,1);
modelParams.estErr = -1000 * ones(nClasses^2,1);

% Mex is faster but tested only on Linux
nSims = length(simDomain);
for iMean = 1:nClasses^2
  [modelParams.lambda(iMean), modelParams.gamma(iMean), ...
    modelParams.estErr(iMean)] = MexComputeLambda( ...
    newMeans(iMean), simDomain, nSims);
end % for iMean

% M-file is slower but more portable
%for iMean = 1:nClasses^2;
%[modelParams.lambda(iMean), modelParams.gamma(iMean), ...
%  modelParams.estErr(iMean)] = computeLambda( ...
%  newMeans(iMean), simDomain);
%end %for iMean

modelParams.lambda = reshape(modelParams.lambda, nClasses, nClasses);
modelParams.gamma = reshape(modelParams.gamma, nClasses, nClasses);
modelParams.estErr = reshape(modelParams.estErr, nClasses, nClasses);
modelParams.meanSim = reshape(newMeans, nClasses, nClasses);

% should we iterate?
%TO DO


% Classify
% NOTE: if same-class SDA works well, put it in its own function. For now
% it's experimental.
% LGC, Nov 16, 2009.
% Update: cross-class SDA works at least as well as, and sometimes better
% than, same-class SDA, so we won't pusue same-class SDA any longer for now.
% Ability to compute same-class SDA is still there, just not used, and it
% does not appear in help text.
% LGC Feb 23, 2010

switch nargout
  case {1,2,3}
    [classEst, logPosteriors] = classifySda(testPointSim, modelParams);
  case {4,5,6}
    [classEst, logPosteriors, sameClassEst, sameClassLogPosteriors, ...
      logMarginalsCell] = classifySda(testPointSim, modelParams);
    logMarginals = logMarginalsCell{1};
  otherwise
    error('Invalid number of output arguments');
end

return %END main function



%% LOCAL FUNCTIONS
% local_determineSimDomain()
function simDomain = local_determineSimDomain(testPointSim, ...
  neighborSimMat, neighborClassLabels, centroidType, varargin)
% The similarity domain consists of all the similarity values observed in
% the training set plus the similarity of the test point to the classes;
% however user can override this definition of similarity domain by
% providing a similarity domain vector as an argument.
switch nargin
  case 5
    simDomain = varargin{:};
  case 4
    centroidModel = computeCentroidModels(neighborSimMat, ...
      neighborClassLabels, centroidType);
    simToCentroids = testPointSim(:, centroidModel.centroidIdx);
    simDomain = unique(unique(neighborSimMat));
    simDomain = [simDomain(:); simToCentroids(:)];
    simDomain = unique(simDomain);
  otherwise
    error('Invalid number of arguments');
end %switch nargin

return %END local_determineSimDomain()