function [classEst, logPosteriors, modelParams, logMarginals] = ...
  classifyPairwiseLocalSda(testPointSim, neighborSimMat, ...
  neighborClassLabels, varargin)
%classifyPairwiseLocalSda - classify a test point with pairwise local SDA.
% Classifies a test sample based on its similarities to a set of other
% samples using loca similarity discriminant analysis (SDA).
%
% USAGE
% TO BE UPDATED - COPIED FROM LOCAL SDA
% [classEst, logPosteriors, modelParams, ...
%    sameClassLogPosteriors, sameClassEst, logMarginals] = classifyLocalSda( ...
%    simVec, simMat, classLabels, centroidType, simDomain)
% simVec: 1-by-M vector of test point similarities to each of the set points.
% simMat: M-by-M matrix of pairwise similarities of all set points
% classLabels: M-by-1 vector of class labels of the points in the set.
% centroidType - string denoting the type of centroid to use. See
%     findCentroid() for a list of allowed centroid types.
% simDomain - vector of all possible similarity values. Optional parameter;
%     if it is not provided, it is constructed from simMat.
% classEst - class estimate for the test point corresponding to simVec.
% logPosteriors - log of the posterior probabilites for the classes.
% modelParams - structure with the computed maximum entropy model
%     parameters used for classification. See computeMaxEntModels for
%     details.
% sameClassLogPosteriors - disabled, returns -Inf.
% sameClassEst - disabled, returns 0.
% logMarginals - G-by-G matrix of the log of the class-conditional marginal
%     probabilities, where G is the number of classes. The output
%     logPosteriors is computed from this matrix.
%
% See also classifySda

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of washington, November 2006.
%$Id:$
%$DateTime:$
%$Author:$

% Initialize stuff
uniqueClassLabels = unique(neighborClassLabels);
nClasses = length(uniqueClassLabels);
logMarginals = -Inf * ones(nClasses,nClasses);
modelParams = initSdaModels;
modelParams.classLabels = uniqueClassLabels;


%% All neighbors are from one class --> classify as that class; done.
if nClasses == 1
  classEst = uniqueClassLabels;
  logPosteriors = 0;
  logMarginals = 0;
  modelParams.classPriors = 1;
  modelParams.classCounts = size(neighborSimMat,1);
  
  %The other fields are empty
  return
end

%% A class has only one neighbor --> classify using nearest-centroid; done.
% REVIEW THIS - MAY NOT BE NECESSARY
%nSamplesPerClass = hist(neighborClassLabels, uniqueClassLabels);
%if any(nSamplesPerClass == 1)
%  [classEst, modelParams] = classifyLocalNearestCentroid( ...
%    testPointSim, neighborSimMat, neighborClassLabels, centroidType);

  
  % For consistency, add these empty fields
  %modelParams.gamma = [];
  %modelParams.lambda = [];
  %modelParams.meanSim = [];
  %modelParams.estErr = [];
  %modelParams.simDomain = [];
  %logMarginals = 0;
  %return

%end % if any(nSamplesPerClass == 1)



%% All other cases --> classify based on MaxEnt models
if nargin == 4
  simDomain = varargin{:};
else
  simDomain = unique([neighborSimMat(:); testPointSim(:)]);
end

modelParams = computePairwiseMaxEntModels(neighborSimMat, ...
  neighborClassLabels, simDomain);
minSim = min(simDomain);
maxSim = max(simDomain);

for gIdx = 1:nClasses
  for hIdx = 1:nClasses
    hClassNeighborIdx = neighborClassLabels == uniqueClassLabels(hIdx);
    testSimsToClassH = testPointSim(hClassNeighborIdx);
    thisLambda = modelParams.lambda(gIdx,hIdx);
    thisGamma = modelParams.gamma(gIdx,hIdx);
    thisMeanSim = modelParams.meanSim(gIdx,hIdx);
    thisExponent = thisLambda * testSimsToClassH;
    % CAREFUL: if thisLambda was -Inf and a testSimToClassH contains a 0, 
    % the corresponding elements in the product are -NaN. Instead we want
    % to keep the -Inf, so we fix this here.
    % LGC, Sept. 2010
    nanProblemIdx = isnan(thisExponent);
    thisExponent(nanProblemIdx) = -Inf;
    thisProb = thisGamma * exp(thisExponent);
    % CAREFUL AGAIN: any degenerate pmfs (Kronecker delta) must be
    % re-introduced here. In computePairwiseMaxEntModels() we manually set
    % lambda = -Inf and gamma = 1 to handle the degenerate pmf case, which
    % happens if meanSim = max (or min) sim. We check for lambda = -Inf, 
    % which sets all probabilities to 0 and we re-introduce a probability = 1
    % for the extremal similarity value. This is consistent with
    % applyMaxEntParameterCorrection(), which does the same thing for
    % sim-to-centroid similarities.
    %LGC, Sept. 2010
    %
    if isinf(thisLambda) && ( ( abs(thisMeanSim - maxSim) < eps ) || ...
        ( abs(thisMeanSim - minSim) < eps )  )
      setProbToOneIdx = abs(testSimsToClassH - thisMeanSim) < eps;
      thisProb(setProbToOneIdx) = 1; % log(1) = 0;
    end
    logMarginals(gIdx, hIdx) = log(mean(thisProb));
  end %for hIdx
end %for gIdx

logPosteriors = sum(logMarginals,2) + log(modelParams.classPriors(:));

% Do all classes have 0 probability? 
if all( isinf(logPosteriors) )
  % THIS IS FOR DEBUGGING - using nearest centroid is consistent with
  % classifyLocalSda(), but it is inconsistent with using all pairwise
  % similarities. This code is for debugging ...
  %centroidParams = computeCentroidModels(neighborSimMat, ...
  %  neighborClassLabels, 'maxSumSim');
  %classEst = classifyNearestCentroid(testPointSim, centroidParams, 'similarity');
  % END DEBUG 
  
  % DO THIS INSTEAD: classify as the class whose mean pairwise similarity
  % to the test point is maximum
   classEst = classifyMaximumMeanSimilarity(testPointSim, neighborClassLabels);
  
else
  [notUsed, classEstIdx] = max(logPosteriors); %#ok
  classEst = modelParams.classLabels(classEstIdx);
end



return %END main function

