function modelParams = computeMaxEntModelsNNSim( simMat, classLabels, ...
  varargin)
%computeMaxEntModelsNNSim - compute maximum entropy model parameters.
% Computes the parameters for mean similarity-constrained maximum
% entropy models, for the k-th nearest neighbor similarity. This function calls
% computeLambda() to compute the maximum entropy model parameters for each
% of the similarity models.
%
% USAGE
% modelParams = computeMaxEntModelsNNSim(simMat, classLabels, simDomain, k)
% siMat: N-by-N matrix of pairwise similarities.
% classLabels: N-by-1 vactor of class labels corresponding to each row of
%     simMat.
% k: optional integer denoting which neighbor to use for the similarity. If not
%     provided, k = 1.
% simDomain: optional vector of possible similarity values. If not
%     provided, it is computed by taking the unique values of simMat. 
% modelParams: structure containing the model parameters. The fields are:
%     * isTrivial: boolean flag, true if only one class is represented in
%     the training data.
%     * classPriors: C-by-1 vector of class prior probabilities, where C is
%     the number of classes represented in classLabels. 
%     * classCounts: C-by-1 vector of class frequency counts for the
%     training set.
%     * classLabels: C-by-1 vector of unique class labels.
%     * lambda: C-by-C matrix of exponent parameters, ordered as
%     [lambda11 lambda12 ... lambda1C;
%      lambda21 lambda22 ... lambda2C;
%      ...
%      lambdaC1 lambdaC2 ... lambdaCC], 
%     where lambda11 is the exponent in the maximum entropy distribution of
%     the similarities to class 1 assuming that a sample is in class 1;
%     lambda 12 is the exponent in the maximum entropy distribution of the
%     similarities to class 2 given that a sample is in class 1; etc.
%     * gamma: C-by-C matrix of normalization constants, arranged the same
%     way as lambda. 
%     * meanSim: C-by-C matrix of mean similarities, arranged the same way
%     as lambda.
%     * estErr: C-by-C matrix containing the estimation errors for
%     * simDomain: vector of possible similarity values.
%     * representativeNeighbor: the value of k used to cpmpute the
%     similarity models.
%
% See also classifySdaNNSim.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, November 2006.


%% Class labels 
uniqueClassLabels = unique(classLabels);
nClasses = length(uniqueClassLabels);

%% Get similarity domain vector
% Is the similarity domain provided or should it be inferred from the
% similarity matrix?
switch nargin
  case 4 % both similarity domain and neighbor index are provided
    similarityDomainVector = varargin{1};
    representativeNeighbor = varargin{2};
  case 3 % similarity domain provided but neighbor index not provided 
    representativeNeighbor = 1;
    similarityDomainVector = varargin{1};
  case 2 % compute
    similarityDomainVector = unique(unique(simMat));
    representativeNeighbor = 1;
  otherwise
    error('Wrong number of input arguments');
end % case
if isempty(similarityDomainVector)
   similarityDomainVector = unique(unique(simMat));
end
if isempty(representativeNeighbor)
  representativeNeighbor = 1;
end

% These are used later
maxSim = max(similarityDomainVector);
minSim = min(similarityDomainVector);


%% Compute centroids, mean similarity to centroids, and lambdas for ...
% If the class-conditional mean is either the maximum or the minimum of the
% similarity domain, then there is only one possible value for the
% similarity. In this case computeLambda() fails (correct result would be
% lambda = -Inf, which results in 0 probability for *all* similarities for
% that class-conditional probability. Instead, we want to assign 0
% probability only if meanSim = maxSim or meanSim = minSim. This is taken
% care of in classifySda(). Here the values of lambda and gamma in this
% special case do not really matter.

% ... reorder columns according to nearest-neighbor ...
[nData, maxNeighbors] = size(simMat);
neighborIdx = findNearNeighbors(simMat, maxNeighbors, 'similarity');
neighborIdx = adjustSelfSimilarityRank(neighborIdx);
classLabelMat = zeros(nData, maxNeighbors);
for iRow = 1:nData
  simMat(iRow,:) = simMat(iRow, neighborIdx(iRow,:));
  classLabelMat(iRow,:) = classLabels(neighborIdx(iRow,:));
end

% ... eliminate self-similar entries ...
neighborIdx(:,1) = []; 
simMat(:,1) = [];
classLabelMat(:,1) = [];

% ... find each sample's nearest neighbor from each class: this amounts to
% finding the first occurrence of each class label in the neighbors' class
% label vector ...
simToNearestNeighbors = zeros(nData, nClasses);
nearestNeighborsIdx = zeros(nData, nClasses);
nearestNeighborsLabels = zeros(nData, nClasses);


for iRow = 1:nData
  for iLabel = 1:nClasses
    idx = find(classLabelMat(iRow,:) == uniqueClassLabels(iLabel));
    if isempty(idx)
      simToNearestNeighbors(iRow,iLabel) = min(similarityDomainVector);
      %nearestNeighborsIdx(iRow, iLabel) = neighborIdx(iRow,idx(actualNeighbor));
      %nearestNeighborsLabels(iRow, iLabel) = classLabelMat(iRow, idx(actualNeighbor));
    else
      actualNeighbor = min(representativeNeighbor, length(idx));
      simToNearestNeighbors(iRow,iLabel) = simMat(iRow, idx(actualNeighbor));
    %nearestNeighborsIdx(iRow, iLabel) = neighborIdx(iRow,idx(actualNeighbor));
    %nearestNeighborsLabels(iRow, iLabel) = classLabelMat(iRow, idx(actualNeighbor));
    end %if
  end % for iLabel
end % for iRow


% ... compute parameters of exponential distributions ...
lambdaVec = zeros(1, nClasses);
gammaVec = zeros(1, nClasses);
estErrVec = zeros(1, nClasses);
classCounts = zeros(1, nClasses);
lambda = zeros(nClasses);
gamma = zeros(nClasses);
estErr = zeros(nClasses);
meanSim = zeros(nClasses);

for iClass = 1:nClasses
  idx = find( classLabels == uniqueClassLabels(iClass) );
  classCounts(iClass) = length(idx);
  thisClassSimMat = simToNearestNeighbors(idx,:);
  meanVec = mean(thisClassSimMat,1);
  
  % DEBUG
  %if numel(meanVec) == 1
  %  keyboard
  %end

  for iNeighbor = 1:nClasses
    if meanVec(iNeighbor) == maxSim || meanVec(iNeighbor) == minSim
      lambdaVec(iNeighbor) = -Inf;
      gammaVec(iNeighbor) = 1;
      estErrVec(iNeighbor) = 0;
    else
      [lambdaVec(iNeighbor), gammaVec(iNeighbor), estErrVec(iNeighbor)] = ...
        computeLambda(meanVec(iNeighbor), similarityDomainVector);
    end %if
  end %for iNeighbor
  lambda(iClass,:) = lambdaVec;
  gamma(iClass,:) = gammaVec;
  estErr(iClass,:) = estErrVec;
  meanSim(iClass,:) = meanVec;
end % for iClass


%% Put results in structure
modelParams = initSdaModels; %initialize to empty fields
modelParams.isTrivial = false;
modelParams.lambda = lambda;
modelParams.gamma = gamma;
modelParams.meanSim = meanSim;
modelParams.estErr = estErr;
modelParams.representativeNeighbor = representativeNeighbor;

% ... remove irrelevant structure fields ...
modelParams = rmfield(modelParams, {'centroidIdx', 'centroidType'});


% ... keep ancillary information too. Note the Laplace correction for the
% priors.
modelParams.classCounts = classCounts;
modelParams.classPriors = ( modelParams.classCounts + 1 )/ ...
  ( sum(modelParams.classCounts) + nClasses);
modelParams.classLabels = uniqueClassLabels;
modelParams.simDomain = similarityDomainVector;


return %END main function