function modelParams = computePairwiseMaxEntModels(simMat, classLabels, ...
  varargin)
%computePairwiseMaxEntModels - compute maximum entropy model parameters.
% Computes the parameters for mean similarity-constrained maximum
% entropy models for any number of classes, using all the apirwise
% similarities as the descriptive statistic.
%
% USAGE
% REVIEW THIS - COPIED FROM OTHER FUNCTION
% modelParams = computePairwiseMaxEntModels(simMat, classLabels, centroidType, ...
%     simDomain)
% siMat: N-by-N matrix of pairwise similarities.
% classLabels: N-by-1 vactor of class labels corresponding to each row of
%     simMat. 
% centroidType: string denoting the type of centroid to use. See
%     findCentroid() for a list of supported centroid types.
% simDomain: optional vector of possible similarity values. If not
%     provided, it is computed by taking the unique values of simMat. 
% modelParams: structure containing the model parameters. The fields are:
%     * isTrivial: boolean flag, true if only one class is represented in
%     the training data.
%     * classPriors: C-by-1 vector of class prior probabilities, where C is
%     the number of classes represented in classLabels. 
%     * classCounts: C-by-1 vector of class frequency counts for the
%     training set.
%     * classLabels: C-by-1 vector of unique class labels.
%     * centroidIdx: 1-by-C vector containing the indices into the 
%     columns of simMat corresponding to the class centroids.
%     * centroidType: string denoting the type of centroid used for the
%     models.
%     * lambda: C-by-C matrix of exponent parameters, ordered as
%     [lambda11 lambda12 ... lambda1C;
%      lamda21 lamda22 ... lambda1C; 
%      ...
%      lambdaC1 lambda C2 ... lambdaCC],
%      where lambda11 is the exponent in the maximum entropy distribution
%      of the similarities to class 1 assuming that a point is in class1;
%      lambda 12 is the exponent in the maximum entropy distribution of the
%      similarities to class 2 given that a point is in class 1; etc.
%     * gamma: C-by-C matrix of normalization constants, arranged the same
%     way as the lambda matrix.
%     * meanSim: C-by-C matrix of mean similarities, arranged as the same
%     way as the lambda matrix, where meanSim11 is the
%     average similarity of the samples in class 1 to the centroid of class
%     1; meanSim12 is the average similarity of the samples in class 1 to
%     the centroid of class 2; etc.
%     * estErr: C-by-C matrix containing the estimation errors as returned
%     by fmisearch(), arranged the same way as the matrix lambda.
%     * simDomain: vector of possible similarity values.
%
% See also initSdaModels, computeSdaModels, classifySda.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, November 2006.
%
% This multi-class code is a revised version of the two-class code.
% Revisions done by Maya Gupta, Dept. of EE, Univ. of Washington
% April 13, 2007.

%% Quick check on number of classes
uniqueClassLabels = unique(classLabels);
nClasses = length(uniqueClassLabels);


%% Get similarity domain vector
% Is the similarity domain provided or should it be inferred from the
% similarity matrix?
switch nargin
  case 3 % provided 
    similarityDomainVector = varargin{1};
  case 2 % compute
    similarityDomainVector = unique(unique(simMat));
  otherwise
    error('Wrong number of input arguments');
end % case

% These are used later
maxSim = max(similarityDomainVector);
minSim = min(similarityDomainVector);
nSims = length(similarityDomainVector);

% Mark each class's data.
% The g-th row of classIdx marks those samples of the g-th class
% with a "1", and all other samples with a "0"
classIdx = false(nClasses, length(classLabels));
for g=1:nClasses % iterate through classes grabbing ID lists
  classIdx(g, :) = classLabels == uniqueClassLabels(g);
end

%% GET MODEL PARAMETERS AND STORE THEM
% Compute mean pairwise similarities between the elements from class g and
% the elements of class h and the corresponding lambdas for the class models.
% IMPORTANT:
% If the class-conditional mean is equal to the maximum or the minimum of the
% similarity domain (this can happen for local SDA), then there is only one
% possible value for the similarity, that is the mean value = max = min. In
% this case computeLambda() gives a garbage result for lambda. The correct
% result would be lambda = -Inf, but this has the problem that *all*
% similarities have 0 class-conditional probability. Instead, the correct 
% pmf would be a discrete delta function centered around the mean
% similarity. For this degenerate case, we assign lambda = -Inf here, and 
% correct the pmf to be the appropriate delta function in classifySda().

%% Put results in structure
modelParams = initSdaModels; %initialize to empty fields
modelParams.isTrivial = false;

% Next there are two for-loops to get the model parameters:
% the outer loop on h specifies the class centroid
% and the inner loop on g specifies which class you are conditioning on

for h=1:nClasses % compute model of all pairwise similarities to class h
    
   for g=1:nClasses % condition on x is in class g
   
        tempSimMat = simMat(classIdx(g,:), classIdx(h,:));
        tempMeanSim = mean(tempSimMat(:));
        if (abs(tempMeanSim - maxSim) < eps) || ...
            (abs(tempMeanSim - minSim) < eps)
          tempLambda = -Inf;
          tempGamma = 1; % either case
          tempEstErr = 0; % either case
        else
          % Mex implementation is 10 times faster but was only tested on
          % Linux and Mac; m-script implementation works on all platforms. Pick the
          % one you want. LGC Dec 1, 2009.
          
          [tempLambda, tempGamma, tempEstErr] = MexComputeLambda( ...
            tempMeanSim, similarityDomainVector, nSims);
          
          %[tempLambda, tempGamma, tempEstErr] = computeLambda(...
           %   tempMeanSim, similarityDomainVector);
          
        end
        % store temp model parameters in right place
        % parameter gh is the ordering.
        modelParams.lambda(g,h) = tempLambda;
        modelParams.gamma(g,h) = tempGamma;
        modelParams.meanSim(g,h) = tempMeanSim;
        modelParams.estErr(g,h) = tempEstErr;
        
   end % for: condition on x in class g
   
end %for: class h sim 


% Store ancillary information too. Note the Laplace correction for the
% priors.
modelParams.classCounts = sum(classIdx,2); % store # of each class
modelParams.classPriors = ( modelParams.classCounts + 1 )/ ...
  ( sum(modelParams.classCounts) + nClasses);
modelParams.classLabels = uniqueClassLabels;
modelParams.simDomain = similarityDomainVector;
modelParams.centroidType = [];
modelParams.centroidIdx = [];
return %END main function

