function centroidIdx = findMultipleCentroids(simMat, initCentroidIdx, ...
  centroidType, varargin)
%findMultipleCentroids - find multiple centroids based on similarity.
% Find the similarity-based cluster centers (centroids) using K-medoids
% algorithm.
%
% USAGE
% centroidIdx = findMultipleCentroids(simMat, initCentroidIdx, ...
%     centroidType, maxIter)
% simMat: matrix of pairwise similarities.
% initCentroidIdx: initial indeces into simMat columns denoting the centroids.
% centroidType: string indicating the desired type of centroid. Allowed
%  types are: 'maxSumSim', 'mostPopular', but tested only for 'maxSumSim'.
% maxIter: optional argument for maximum number of iterations; default = 25.
% centroidIdx: indeces into simMat columns denoting the centroids.

% Written by Luca Cazzanti, engineer
% Applied Physics Laboratory, University of Washington, Feb 2007.
%$Id:$
%$DateTime:$
%$Author:$

%% Check trivial case of only one centroid
nInitCentroids = length(initCentroidIdx);
if nInitCentroids == 1
  centroidIdx = findCentroid(simMat, centroidType);
  return
end

%% Use k-medoids with similarity.
% There are two steps to the algorithm:
% a) Given the initial centroids, assign each sample to the centroid to
% which it is most similar. Ties are broken randomly, except that each
% centroid is explicitly assigned to itself, even if it is equally similar
% to another centroid. The result is that the samples are partinioned in
% mutually exclusive clusters.
% b) Update the centroids by selecting one sample from each cluster
% according to centroidType. For example, for centroidType = 'maxsumSim',
% each cluster produces a centroid that is the element with maximum sum
% similarity to all other elements in the cluster. In case of ties with the
% current centroid, the current centroid is retained (no update).

switch nargin
  case 3
    maxIter = 25;
  case 4
    maxIter = varargin{1};
  otherwise
    error('Invalid number of input arguments')
end % switch nargin

sortType = 'similarity';
stopCondition = false;
iterCount = 1;
while ( stopCondition == false && iterCount < maxIter )
%  while ( iterCount < maxIter )
  % a) Assign samples to centroids ...
  simToCentroidsMat = simMat(:, initCentroidIdx);
  [nearestCentroidIdx, sortedSimToCentroidsMat] = findNearNeighbors( ...
    simToCentroidsMat, length(initCentroidIdx), sortType);
  
  % ... randomly break ties in similarities: if a sample is equally similar to
  % more than one centroid, randomly assign it to one of them ...
  for iRow = 1:size(sortedSimToCentroidsMat,1)
    tiedColIdx = find( sortedSimToCentroidsMat(iRow,:) == sortedSimToCentroidsMat(iRow,1) );
    randomIdx = randperm(max(tiedColIdx));
      nearestCentroidIdx(iRow,tiedColIdx) = nearestCentroidIdx(iRow,randomIdx);
  end
  nearestCentroidIdx = nearestCentroidIdx(:,1);
  
  % ... however, a centroid must always be assigned to itself.
  for iCentroid = initCentroidIdx(:).';
    nearestCentroidIdx(iCentroid) = find(initCentroidIdx == iCentroid);
  end
  
  % b) Update the centroids: go through each cluster
  uniqueNearestCentroidsIdx = unique(nearestCentroidIdx);
  nUniqueNearestCentroids = length(uniqueNearestCentroidsIdx);
  centroidIdx = zeros(1, nUniqueNearestCentroids);
  for iCentroid = 1:nUniqueNearestCentroids
    assignedToThisCentroidIdx = find( ...
      nearestCentroidIdx == uniqueNearestCentroidsIdx(iCentroid) );
    assignedToThisCentroidSimMat =  ...
      simMat(assignedToThisCentroidIdx, assignedToThisCentroidIdx);
    
    currentCentroidIdx = find(assignedToThisCentroidIdx == ...
      initCentroidIdx(uniqueNearestCentroidsIdx(iCentroid)));
    centroidIdx(iCentroid) = local_findCentroidBreakTies( ...
      assignedToThisCentroidSimMat, currentCentroidIdx, centroidType);
    
    % old way of finding centroid which did not take into account possible
    % tie with current centroid:
    %centroidIdx(iCentroid) = findCentroid(assignedToThisCentroidSimMat, ...
    % centroidType);
    
    % Convert centroid index into submatrix --> index into original matrix
    centroidIdx(iCentroid) =  assignedToThisCentroidIdx(centroidIdx(iCentroid));
  end %for iCentroid
  
  % Are we done yet with the iterations?
  if (nUniqueNearestCentroids == nInitCentroids)
    stopCondition = all(initCentroidIdx(:) == centroidIdx(:));
  else
    stopCondition = false;
  end
  iterCount = iterCount + 1;
  
  % Update centroid values and go to next iteration
  initCentroidIdx = centroidIdx;
  
end %while stopCondition

return

%% LOCAL FUNCTIONS

function centroidIdx = local_findCentroidBreakTies(simMat, ...
  currentCentroidIdx, centroidType)
%Update the centroid from a similarity matrix, breaking ties as follows:
% if the set of maximum-sum similarity candidate centroids contains the
% current centroid, assign the current centroid to the updated centroid,
% that is no change; otherwise, assign the first (lowest index) candidate
% centroid to the new centroid, which is what is done in
% findCentroidMaxSum(). 
switch centroidType
  case 'maxSumSim'
    sumSim = sum(simMat,2);
    maxSumSim = max(sumSim);
    candidateCentroidIdx = find(sumSim == maxSumSim);
    try
    if length(candidateCentroidIdx) == size(simMat,1) % all same sim
      tmp = randperm(length(candidateCentroidIdx));
      centroidIdx = tmp(1);
    elseif any( candidateCentroidIdx == currentCentroidIdx )
      centroidIdx = currentCentroidIdx;
    else
      centroidIdx = candidateCentroidIdx(1);
    end
    catch
      keyboard
    end
    return

  case 'mostPopular' % TO DO: tie-breaking for most-popular
    centroidIdx = findCentroidMostPopular(simMat); 
  otherwise
    error(['Centroid type ' centroidType ' not supported']);
end %switch

return
  
